/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* alm
 *
 * Copyright (C) 2012 Canonical
 *		  Authored by: Didier Roche <didrocks@ubuntu.com>
 * Copyright (C) 2013 Manish Sinha <manishsinha@ubuntu.com>
 *
 * alm is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * alm is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.";
 */

using Gtk;

namespace Alm {

	public class SearchResultsWidget : Gtk.Box {

		private GLib.Settings gp_settings;
		private const string REMOTE_CONTENT_KEY = "remote-content-search";

		public enum RemoteContent
		{
			ALL,
			NONE,
		}

		Switch commercial_suggestion;

		public SearchResultsWidget () {
			Object (orientation: Orientation.VERTICAL);
			this.spacing = 0;
			this.set_border_width(12);

			this.gp_settings = new GLib.Settings ("com.canonical.Unity.Lenses");
			this.gp_settings.bind (REMOTE_CONTENT_KEY, this, "remote-content-search", SettingsBindFlags.DEFAULT);

			this.set_up_ui ();
		}

		public RemoteContent remote_content_search { get; set; default = RemoteContent.ALL; }


		public void set_up_ui () {
			var top_box = new Box(Orientation.HORIZONTAL, 21);
			top_box.set_margin_top(9);
			this.pack_start(top_box, false, false);

			var text_box = new Box(Orientation.VERTICAL, 0);
			var header = new Label("");
			header.set_markup("<b>%s</b>".printf(_("When searching in the Dash:")));
			header.set_alignment(0, (float)0.5);
			header.set_padding(0, 0);

			text_box.pack_start(header, false, false);

			var label = new Label(null);
			label.set_markup("%s".printf(_("Include online search results")));
			label.set_line_wrap(true);
			label.set_line_wrap_mode(Pango.WrapMode.WORD);
			label.set_alignment(0, 0.5f);
			text_box.pack_start(label, false, false, 6);
			top_box.pack_start(text_box, false, false);

			commercial_suggestion = new Switch();
			if (this.remote_content_search == RemoteContent.ALL)
				commercial_suggestion.set_active(true);
			commercial_suggestion.notify["active"].connect(on_commercial_suggestion_activated);

			var temp_box = new Box(Orientation.VERTICAL, 0);
			temp_box.pack_start(new Label(""), true, true);
			temp_box.pack_start(commercial_suggestion, false, false);
			temp_box.pack_start(new Label(""), true, true);
			top_box.pack_end(temp_box, false, false);

            var unity_scope_widget = new UnityScopeWidget();
            this.pack_start(unity_scope_widget, true, true);
		}

		public void on_commercial_suggestion_activated() {
			if (this.commercial_suggestion.get_active())
			{
				this.remote_content_search = RemoteContent.ALL;
			}
			else
			{
				this.remote_content_search = RemoteContent.NONE;
			}
		}
	}

	public class UnityScopeWidget : Gtk.Box {

        private TreeView scopes_view;
        private ListStore scopes_store;
	    public UnityScopeWidget () {
		    Object (orientation: Orientation.VERTICAL);
		    this.spacing = 0;
		    this.set_border_width(12);

		    this.set_up_ui ();
	    }

        public void set_up_ui() {
            this.scopes_store = new ListStore(2, typeof(string), typeof(string));
            this.scopes_view = new TreeView.with_model(this.scopes_store);
            this.scopes_view.set_headers_visible(false);

            var column_text = new TreeViewColumn();
            column_text.set_title("Scope Name");
            this.scopes_view.append_column(column_text);

            var name_rend = new CellRendererText();
            name_rend.set_property("ellipsize", Pango.EllipsizeMode.END);
            column_text.pack_start(name_rend, true);
            column_text.add_attribute(name_rend, "markup", 0);
            column_text.set_resizable(true);

            var scroll = new ScrolledWindow(null, null);
            scroll.add(this.scopes_view);
            scroll.set_policy(PolicyType.NEVER, PolicyType.AUTOMATIC);
            scroll.set_shadow_type(ShadowType.IN);
            scroll.set_border_width(1);
            this.pack_start(scroll);

            this.fetch_data();
        }

        private void fetch_data() {
            string[] xdg_data_dirs = this.get_xdg_data_dirs();
            var all_scopes =  this.get_all_scopes(xdg_data_dirs);
        }

        private ScopeData get_scope_data(string scope_filename) {
            return null;
        }

        private HashTable<string, GenericArray<string>> get_all_scopes(string[] xdg_data_dirs) {
            var all_scopes = new HashTable<string, GenericArray<string>>(str_hash, str_equal);

            foreach(string xdg_data in xdg_data_dirs) {
                string dir_path = Path.build_filename(xdg_data, "unity", "scopes");
                if(!FileUtils.test(dir_path, FileTest.EXISTS))
                    continue;

                GenericArray<string> scopes = this.get_scope_files(dir_path);
                all_scopes.insert(dir_path, scopes);
                for(int i=0; i < scopes.length; i++) {
                    stdout.printf("%s\n", scopes[i]);
                }
            }
            return all_scopes;
        }

        private GenericArray<string> get_scope_files(string dir_path) {
            GenericArray<string> scopes = new GenericArray<string>();
            var dir = Dir.open(dir_path);
            string? name = null;
            while ((name = dir.read_name ()) != null) {
		        string path = Path.build_filename (dir_path, name);
                if(FileUtils.test(path, FileTest.IS_REGULAR) && path.has_suffix(".scope")) {
                    scopes.add(path);
                }
                if(FileUtils.test(path, FileTest.IS_DIR)) {
                    var sub_scopes = this.get_scope_files(path);
                    for(int i=0;i<sub_scopes.length;i++)
                        scopes.add(sub_scopes[i]);
                }
            }
            return scopes;
        }

        private string[] get_xdg_data_dirs() {
            string xdg_data = Environment.get_variable("XDG_DATA_DIRS");
            if(xdg_data == null || xdg_data.strip().length == 0)
                xdg_data = "/usr/share";
            return xdg_data.split(":");
        }
    }

    private class ScopeData {
        public ScopeData(string scope_filepath) {

        }
    }
}
